/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CreatePlainDiskImageDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"

#include "Pline/FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

CreatePlainDiskImageDialog::CreatePlainDiskImageDialog(
		BaseObjectType* cobject,
		const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Create Plain Disk Image"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpvecVolumeConfigLabel = 0;
	
	UtilGui::mapCreatePlainDiskImageDlg(refGlade, gtCreatePlainDiskImageDlg);	
	
	gtCreatePlainDiskImageDlg.poCreateButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreatePlainDiskImageDialog::onCreate));

	gtCreatePlainDiskImageDlg.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreatePlainDiskImageDialog::onCancel));
		
	gtCreatePlainDiskImageDlg.poSaveAsBrowseButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreatePlainDiskImageDialog::onSaveAs));
		
	Gtk::ComboBoxText* pocmbFSys = manage(new Gtk::ComboBoxText);
			
	gtCreatePlainDiskImageDlg.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtCreatePlainDiskImageDlg.poFSysComboBox = pocmbFSys;
	
	Gtk::ComboBoxText* pocmbPlaceAfter = manage(new Gtk::ComboBoxText);
			
	gtCreatePlainDiskImageDlg.poPlaceAfterVBox->pack_start(*pocmbPlaceAfter, false, false);
	gtCreatePlainDiskImageDlg.poPlaceAfterComboBox = pocmbPlaceAfter;
	
	show_all_children();
}

//==============================================================================

CreatePlainDiskImageDialog::~CreatePlainDiskImageDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
}

//==============================================================================

void CreatePlainDiskImageDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtCreatePlainDiskImageDlg.poFSysComboBox->clear_items();
	
	Pline::FSysUtil::getMkFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtCreatePlainDiskImageDlg.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void CreatePlainDiskImageDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void CreatePlainDiskImageDialog::onCreate()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void CreatePlainDiskImageDialog::onSaveAs()
{
	gpoDialogHelper->onSaveAs();
}

//==============================================================================

void CreatePlainDiskImageDialog::setData(  	VolumeConfig *poVolumeConfig,
											std::vector<Glib::ustring> *pvecVolumeConfigLabel,
											unsigned long long *pullVolumeSize,
											bool *pbIsFullAllocation)
{
	gpoVolumeConfig = poVolumeConfig;
	gpvecVolumeConfigLabel = pvecVolumeConfigLabel;
	
	buildFSys();
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	// mount
	gpoDialogHelper->setMkFSysComboBox(gtCreatePlainDiskImageDlg.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setPlaceAfterComboBox(gtCreatePlainDiskImageDlg.poPlaceAfterComboBox);
	gpoDialogHelper->setPlaceAfterList(gpvecVolumeConfigLabel);
	gpoDialogHelper->setSaveAsEntry(gtCreatePlainDiskImageDlg.poSaveAsEntry);
	gpoDialogHelper->setVolumeLabelEntry(gtCreatePlainDiskImageDlg.poVolumeLabelEntry);

	// new disk image
	gpoDialogHelper->setVolumeSizeEntry(gtCreatePlainDiskImageDlg.poVolumeSizeEntry,
										pullVolumeSize);
	
	gpoDialogHelper->setSizeRadioButton(gtCreatePlainDiskImageDlg.poMRadioButton,
										gtCreatePlainDiskImageDlg.poGRadioButton);
	
	gpoDialogHelper->setAllocRadioButton(	gtCreatePlainDiskImageDlg.poAllocFullRadioButton,
											gtCreatePlainDiskImageDlg.poAllocCompactRadioButton,
											pbIsFullAllocation);
										
	gpoDialogHelper->buildPlaceAfter();
		
	updateData(true);
}
					
//==============================================================================

void CreatePlainDiskImageDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateCreateDiskImageData(bToScreen);
}

//==============================================================================

bool CreatePlainDiskImageDialog::validate()
{
	if (!gpoDialogHelper->validateCreateDiskImageData())
		return false;
		
	return true;
}

//==============================================================================
