/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "EnlargeVolumeDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"
#include "Util.h"
#include "VolumeDialog.h"

#include "Pline/FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

EnlargeVolumeDialog::EnlargeVolumeDialog(
		BaseObjectType* cobject,
		const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Enlarge Volume"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	
	UtilGui::mapEnlargeVolumeDlg(refGlade, gtEnlargeVolumeDlg);	
	
	gtEnlargeVolumeDlg.poEnlargeButton->signal_clicked().connect(
		sigc::mem_fun(*this, &EnlargeVolumeDialog::onEnlarge));

	gtEnlargeVolumeDlg.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &EnlargeVolumeDialog::onCancel));
}

//==============================================================================

EnlargeVolumeDialog::~EnlargeVolumeDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
}

//==============================================================================

void EnlargeVolumeDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void EnlargeVolumeDialog::onEnlarge()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void EnlargeVolumeDialog::setData(	VolumeConfig *poVolumeConfig,
									unsigned long long *pullNewVolumeSize,
									const unsigned long long ullCurrVolumeSize)
{
	gpoVolumeConfig = poVolumeConfig;
	
	gpullNewVolumeSize = pullNewVolumeSize;
	gullCurrVolumeSize = ullCurrVolumeSize;
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	gpoDialogHelper->setSaveAsEntry(gtEnlargeVolumeDlg.poSaveAsEntry);
	gpoDialogHelper->setVolumeLabelEntry(gtEnlargeVolumeDlg.poVolumeLabelEntry);
			
	gpoDialogHelper->setVolumeSizeEntry(gtEnlargeVolumeDlg.poNewVolumeSizeEntry,
										pullNewVolumeSize);
	
	gpoDialogHelper->setSizeRadioButton(gtEnlargeVolumeDlg.poMRadioButton,
										gtEnlargeVolumeDlg.poGRadioButton);

	updateData(true);
}
					
//==============================================================================

void EnlargeVolumeDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateEnlargeVolumeData(bToScreen);
	
	if (bToScreen)
	{
		double dMpx;
		Glib::ustring stMpx;
				
		if (gullCurrVolumeSize <= 1024*1024*1024)
		{
			dMpx = 1024*1024;
			stMpx = "M";
		}
		else
		{
			dMpx = 1024*1024*1024;
			stMpx = "G";
		}
			
		gtEnlargeVolumeDlg.poVolumeSizeEntry->set_text(
			Util::compose("%0.2f%s", (double)gullCurrVolumeSize/dMpx, stMpx.c_str()));
		
		gtEnlargeVolumeDlg.poNewVolumeSizeEntry->grab_focus();
	}
}

//==============================================================================

bool EnlargeVolumeDialog::validate()
{
	if (*gpullNewVolumeSize <= gullCurrVolumeSize
		|| *gpullNewVolumeSize <= 0)
	{
		VolumeDialog::tConfig tConfigItem;
		VolumeDialog::initConfig(tConfigItem);
	
		tConfigItem.bWaitData = false;
		tConfigItem.stLabel = gpoVolumeConfig->getData().getLabel();
		tConfigItem.stMsg = _("Invalid size.");
		
		VolumeDialog oDlg(tConfigItem);
	
		return false;
	}
	
	VolumeDialog::tConfig tConfigItem;
	VolumeDialog::initConfig(tConfigItem);
			
	tConfigItem.bAutoRun = false;
	tConfigItem.bWaitData = false;
	tConfigItem.bYesNo = true;
	tConfigItem.bCancelButton = true;
	tConfigItem.bCancelButtonHasFocus = true;
	tConfigItem.stLabel = gpoVolumeConfig->getData().getLabel();
	tConfigItem.stMsg = Util::compose(	_("This operation may corrupt your data in %s irrevocably, continue?"),
										gpoVolumeConfig->getData().getSrc().c_str());
			
	VolumeDialog oDlg(tConfigItem);
	
	if (oDlg.run() != Gtk::RESPONSE_OK)
		return false;
	
	return true;
}

//==============================================================================
