/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "InitVolumeDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"
#include "Util.h"
#include "VolumeDialog.h"

#include "Pline/FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

InitVolumeDialog::InitVolumeDialog(
		BaseObjectType* cobject,
		const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Initialize Volume"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpoKeyFileFileChooser = 0;
	
	UtilGui::mapInitVolumeDlg(refGlade, gtInitVolumeDlg);	
	
	gtInitVolumeDlg.poInitButton->signal_clicked().connect(
		sigc::mem_fun(*this, &InitVolumeDialog::onInit));

	gtInitVolumeDlg.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &InitVolumeDialog::onCancel));
			
	Gtk::ComboBoxText* pocmbFSys = manage(new Gtk::ComboBoxText);
			
	gtInitVolumeDlg.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtInitVolumeDlg.poFSysComboBox = pocmbFSys;
		
	gtInitVolumeDlg.poEncryptedCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &InitVolumeDialog::onEncrypt));
		
	gtInitVolumeDlg.poUsePasswordRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &InitVolumeDialog::onUsePassword));
	
	gtInitVolumeDlg.poUseKeyFileRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &InitVolumeDialog::onUseKeyFile));
		
	Gtk::ComboBoxText* pocmbCipher = manage(new Gtk::ComboBoxText);
			
	gtInitVolumeDlg.poCipherVBox->pack_start(*pocmbCipher, false, false);
	gtInitVolumeDlg.poCipherComboBox = pocmbCipher;
	
	Gtk::ComboBoxText* pocmbCBC = manage(new Gtk::ComboBoxText);
			
	gtInitVolumeDlg.poCBCVBox->pack_start(*pocmbCBC, false, false);
	gtInitVolumeDlg.poCBCComboBox = pocmbCBC;
	
	Gtk::ComboBoxText* pocmbKeySize = manage(new Gtk::ComboBoxText);
			
	gtInitVolumeDlg.poKeySizeVBox->pack_start(*pocmbKeySize, false, false);
	gtInitVolumeDlg.poKeySizeComboBox = pocmbKeySize;
	
	show_all_children();
}

//==============================================================================

InitVolumeDialog::~InitVolumeDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
	
	if (gpoKeyFileFileChooser)
		delete gpoKeyFileFileChooser;
}

//==============================================================================

bool InitVolumeDialog::askConfirm(VolumeConfig *poVolumeConfig)
{
	VolumeDialog::tConfig tConfigItem;
	VolumeDialog::initConfig(tConfigItem);
			
	tConfigItem.bAutoRun = false;
	tConfigItem.bWaitData = false;
	tConfigItem.bYesNo = true;
	tConfigItem.bCancelButton = true;
	tConfigItem.bCancelButtonHasFocus = true;
	tConfigItem.stLabel = poVolumeConfig->getData().getLabel();
	tConfigItem.stMsg = Util::compose(	_("This will overwrite data on %s irrevocably, continue?"),
										poVolumeConfig->getData().getSrc().c_str());
			
	VolumeDialog oDlg(tConfigItem);
			
	return (!(oDlg.run() != Gtk::RESPONSE_OK));
}
	
//==============================================================================

void InitVolumeDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtInitVolumeDlg.poFSysComboBox->clear_items();
	
	Pline::FSysUtil::getMkFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtInitVolumeDlg.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void InitVolumeDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void InitVolumeDialog::onEncrypt()
{
	gpoDialogHelper->onEncrypt();
}

//==============================================================================

void InitVolumeDialog::onInit()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void InitVolumeDialog::onUseKeyFile()
{
	gpoDialogHelper->onUseKeyFile();
}

//==============================================================================

void InitVolumeDialog::onUsePassword()
{
	gpoDialogHelper->onUsePassword();
}

//==============================================================================

void InitVolumeDialog::setData(VolumeConfig *poVolumeConfig,
							Glib::ustring *pstCipher,
							Glib::ustring *pstCBC,
							Glib::ustring *pstKeySize)
{
	gpoVolumeConfig = poVolumeConfig;
	
	buildFSys();
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	if (gpoKeyFileFileChooser)
		delete gpoKeyFileFileChooser;
	
	gpoKeyFileFileChooser = new Gtk::FileChooserButton;
	
	gtInitVolumeDlg.poKeyFileVBox->pack_start(*gpoKeyFileFileChooser, false, false);
	gtInitVolumeDlg.poKeyFileFileChooser = gpoKeyFileFileChooser;
	
	show_all_children();
	
	gpoDialogHelper->setSaveAsEntry(gtInitVolumeDlg.poSaveAsEntry);
	gpoDialogHelper->setMkFSysComboBox(gtInitVolumeDlg.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setVolumeLabelEntry(gtInitVolumeDlg.poVolumeLabelEntry);
	
	gpoDialogHelper->setEncryptedCheckButton(gtInitVolumeDlg.poEncryptedCheckButton);
	gpoDialogHelper->setKeyFileFileChooser(gtInitVolumeDlg.poKeyFileFileChooser);
	gpoDialogHelper->setUseKeyFileRadioButton(gtInitVolumeDlg.poUseKeyFileRadioButton);
	gpoDialogHelper->setUsePasswordRadioButton(gtInitVolumeDlg.poUsePasswordRadioButton);
	
	gpoDialogHelper->setCipherComboBox(	gtInitVolumeDlg.poCipherComboBox,
										pstCipher);
	gpoDialogHelper->setCBCComboBox(gtInitVolumeDlg.poCBCComboBox,
									pstCBC);
	gpoDialogHelper->setKeySizeComboBox(gtInitVolumeDlg.poKeySizeComboBox,
										pstKeySize);
	
	gpoDialogHelper->buildCiphers();
		
	updateData(true);
}
					
//==============================================================================

void InitVolumeDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateInitVolumeData(bToScreen);
}

//==============================================================================

bool InitVolumeDialog::validate()
{
	if (!askConfirm(gpoVolumeConfig))
		return false;
	
	if (!gpoDialogHelper->validateMountData(true))
		return false;
		
	if (gpoVolumeConfig->getCryptData())
	{
		if (!gpoDialogHelper->validateCryptData())
			return false;
	}
			
	return true;
}

//==============================================================================
