/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CryptFilter.h"
#include "CryptUtil.h"
#include "../QDebug.h"
#include "../eMountSys.h"
#include "../UtilSh.h"
#include "../Util.h"
#include "../VolumeDialog.h"

#include <assert.h>

namespace Pline
{

//==============================================================================

CryptFilter::CryptFilter()
: MountBaseFilter()
{
	gpoCryptData = 0;
}

//==============================================================================

CryptFilter::~CryptFilter()
{
}

//==============================================================================

void CryptFilter::askData()
{	
	assert(gpoCryptData);
	
	Glib::ustring stPasswd;
	
	if (gstPasswd.empty() && gpoCryptData->getKeyFile().empty())
	{
		// sh mode
		if (eMountSys::getInstance().getRunMode() == eMountSys::kRunSh)
		{
			while (stPasswd.empty())
			{
				stPasswd = UtilSh::askPassword();
			
				if (stPasswd.empty() && UtilSh::askCancel())
				{
					cancelDataReq();
					return;
				}
			
				if (!stPasswd.empty())
					Util::base64Enc(stPasswd, &gstPasswd);
			}
		}
		// gui mode
		else if (eMountSys::getInstance().getRunMode() == eMountSys::kRunGui)
		{
			VolumeDialog::tConfig tConfigItem;
			VolumeDialog::initConfig(tConfigItem);
			
			tConfigItem.bAutoRun = false;
			tConfigItem.bWaitData = false;
			tConfigItem.bAskPassword = true;
			tConfigItem.stLabel = gpoCryptData->getLabel();
			tConfigItem.stSrc = gpoCryptData->getSrc();
			tConfigItem.stCommand = getFilterName();
			
			VolumeDialog oDlg(tConfigItem);
			
			if (oDlg.run() != Gtk::RESPONSE_OK)
			{
				oDlg.hide();
			
				while (Gtk::Main::events_pending()) // force redraw
					Gtk::Main::iteration();
				
				cancelDataReq();
				return;
			}

			gstPasswd = oDlg.getPassword();
			
			oDlg.hide();
			
			while (Gtk::Main::events_pending()) // force redraw
				Gtk::Main::iteration();

		}
	}
	
	Filter::askData();
}

//==============================================================================

bool CryptFilter::attach()
{	
	assert(gpoCryptData);
	assert(gpoDestData);
	
	Glib::ustring stDev;
	Glib::ustring stPasswd;
	
	// reset error
	gstLastError.clear();
	
	// reset password
	gstPasswd.clear();
	
	if (isAttached())
		return false;
		
	// ask password
	if (gpoCryptData->getKeyFile().empty())
	{
		requestData();	

		if (!waitData())
			return false;
			
		stPasswd = gstPasswd;
	}
	
	int ci = 0;
	
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{
		if (CryptUtil::setCrypt(gpoCryptData->getSrc(),
								stPasswd,
								gpoCryptData->getKeyFile(),
								&stDev,
								&gstLastError))
		{
			gpoDestData->setSrc(stDev);
	
			gstPasswd.clear();
			
			return true;
		}
		
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}

	gstPasswd.clear();
	
	return false;
}

//==============================================================================

bool CryptFilter::detach()
{
	assert(gpoDestData);
	
	int ci = 0;
		
	// reset error
	gstLastError.clear();
	
	if (!isAttached())
		return false;
		
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{
		if (CryptUtil::detachDev(gpoDestData->getSrc(), &gstLastError))
		{
			gpoDestData->setSrc("");
			
			return true;
		}
				
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}
				
	return false;
}

//==============================================================================

Data* CryptFilter::getDestData()
{
	return gpoDestData;
}

//==============================================================================

bool CryptFilter::isAttached()
{
	assert(gpoCryptData);

	if (CryptUtil::findDev(gpoCryptData->getSrc()))
		return true;

	return false;
}

//==============================================================================

bool CryptFilter::probe()
{
	assert(gpoCryptData);
	assert(gpoDestData);

	Glib::ustring stDevMapper;

	if (CryptUtil::findDev(gpoCryptData->getSrc(), &stDevMapper))
	{
		gpoDestData->setSrc(stDevMapper);
		return true;
	}
	else
		gpoDestData->setSrc("");

	return false;
}

//==============================================================================

void CryptFilter::setSrcData(CryptData *poData)
{
	gpoCryptData = poData;

	Filter::setSrcData(poData);
}

//==============================================================================

void CryptFilter::setDestData(Data *poData)
{
	Filter::setDestData(poData);
}

//==============================================================================

} // Pline
