/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CRYPTUTIL_H
#define CRYPTUTIL_H

#include <gtkmm.h>

namespace Pline
{

class CryptUtil
{
public:
	CryptUtil();
	~CryptUtil();

	static bool detachDev(  const Glib::ustring& stDevMapper,
							Glib::ustring *pstErrMsg = 0);
	
	static bool findDev(const Glib::ustring& stDevice,
						Glib::ustring *pstDevMapper = 0);
	
	static bool formatCrypt(const Glib::ustring& stDev, 
							const Glib::ustring& stPasswd,
							const Glib::ustring& stKeyFile,
							const Glib::ustring& stCipher,
							const Glib::ustring& stCBC,
                            const Glib::ustring& stKeySize,
                            Glib::ustring *pstErrMsg = 0);

	static bool getStatus(	const Glib::ustring& stDev,
							Glib::ustring *pstCipher,
							Glib::ustring *pstKeySize);
							
	static bool setCrypt(   const Glib::ustring& stDevice,
							const Glib::ustring& stPasswd,
							const Glib::ustring& stKey,
							Glib::ustring *pstDevMapper,
							Glib::ustring *pstErrMsg = 0);
	
	// kernel mods
	static bool getAvailCipherMod(std::vector<Glib::ustring> *pvecMod);
	static const Glib::ustring getCipherModPath();
	
	// ciphers
	static void getCBCList(std::vector<Glib::ustring> *pvecCBCMode);
	static bool getCipherList(std::vector<Glib::ustring> *pvecCipher);
	static void getKeySizeList(std::vector<Glib::ustring> *pvecKeySize);
	
	// validate
	static bool isValidDevice( const Glib::ustring& stDevice, 
								Glib::ustring *pstErrMsg = 0);
	static bool isValidKey( const Glib::ustring& stKey, 
							Glib::ustring *pstErrMsg = 0);
														
protected:

private:
	typedef struct _DEVMAPPER_STATUS_
	{
		Glib::ustring stDevMapper;
		Glib::ustring stCipher;
		Glib::ustring stKeySize;
		Glib::ustring stDevice;
		Glib::ustring stOffest;
		Glib::ustring stSize;
		Glib::ustring stMode;
	}
	tDevMapperStatus;
	
	static bool getDeviceStatus(const Glib::ustring stDevMapper,
								tDevMapperStatus *ptDevMapperStatus);
	static void getDevMapperList(std::vector<tDevMapperStatus> *pvec);
	static const Glib::ustring getDevMapperName(const Glib::ustring stDevice);
};

} // Pline		
#endif /* CRYPTUTIL_H */
