/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "LoopFilter.h"
#include "LoopUtil.h"
#include "../QDebug.h"
#include "../eMountSys.h"
#include "../UtilSh.h"

#include <assert.h>

namespace Pline
{

//==============================================================================

LoopFilter::LoopFilter()
: MountBaseFilter()
{
	gpoLoopData = 0;
}

//==============================================================================

LoopFilter::~LoopFilter()
{
}

//==============================================================================

void LoopFilter::askData()
{	
	Filter::askData();
}

//==============================================================================

bool LoopFilter::attach()
{
	assert(gpoLoopData);
	assert(gpoDestData);
	
	int ci = 0;
	Glib::ustring stDev;
	
	// reset error
	gstLastError.clear();
	
	if (isAttached())
		return false;
		
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{
		bool bDevFound = false;
				
		if (LoopUtil::findDev(gpoLoopData->getSrc(), &stDev))
			bDevFound = true;
		
		if (!bDevFound && LoopUtil::findUnusedDev(&stDev))
			bDevFound = true;

		if (!bDevFound && LoopUtil::createDev(&stDev))
			bDevFound = true;
		
		if (bDevFound)
		{
			bool bReadOnly = false;
			
			if (LoopUtil::setLoop(	stDev, 
									gpoLoopData->getSrc(),
									&bReadOnly,
									&gstLastError))
			{
				gpoDestData->setSrc(stDev);
					
				return true;
			}
		}
		
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}
	
	gstLastError = _("Could not find loop device.");
		
	return false;
}

//==============================================================================

bool LoopFilter::detach()
{
	assert(gpoDestData);
	
	int ci = 0;
	
	// reset error
	gstLastError.clear();
	
	if (!isAttached())
		return false;
	
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{
		if (LoopUtil::detachDev(gpoDestData->getSrc(), &gstLastError))
		{
			gpoDestData->setSrc("");
					
			return true;
		}
		
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}
				
	return false;
}

//==============================================================================

bool LoopFilter::isAttached()
{
	assert(gpoLoopData);

	if (!LoopUtil::findDev(gpoLoopData->getSrc()))
		return false;	
	
	return true;
}

//==============================================================================

bool LoopFilter::probe()
{
	assert(gpoLoopData);
	assert(gpoDestData);
	
	Glib::ustring stDev;

	if (LoopUtil::findDev(gpoLoopData->getSrc(), &stDev))
	{
		gpoDestData->setSrc(stDev);
		return true;
	}
	else
		gpoDestData->setSrc("");
	
	return false;
}



//==============================================================================

void LoopFilter::setSrcData(LoopData *poData)
{
	gpoLoopData = poData;

	Filter::setSrcData(poData);
}

//==============================================================================

void LoopFilter::setDestData(Data *poData)
{
	Filter::setDestData(poData);
}

//==============================================================================

} // Pline
