/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MkDiskImagePline.h"
#include "QDebug.h"
#include "../Util.h"

#include "CloneData.h"
#include "CopyFilter.h"
#include "CryptFilter.h"
#include "LoopFilter.h"
#include "MountFilter.h"
#include "MkFSysFilter.h"
#include "MkCryptFilter.h"

#include <iostream>
#include <iomanip>
#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

MkDiskImagePline::MkDiskImagePline( const eDataMode eDataModeItem,
									VolumeConfig *poVolumeConfig,
									const unsigned long long ullVolumeSize,
									const bool bUseFullAllocation,
									const Glib::ustring& stCipher,
									const Glib::ustring& stCBC,
									const Glib::ustring& stKeySize)
: ProcessBasePline(eDataModeItem, poVolumeConfig)
{
	assert(goVolumeConfig.getMountData());
	
	gfCurrentSpeed = 0;
	giPctDone = 0;
	giCopyPctDone = 0;
	
	gbCopying  = false;
	gbCryptFormat = false;
	gbCryptOpen = false;
	gbMkFSys = false;
	
	gpoCryptDetachFilter = 0;
	gpoLoopDetachFilter = 0;
	
	bool bUseCrypt = 
		(goVolumeConfig.getCryptData() && goVolumeConfig.getCryptData()->getIsEncrypted());
	//
	// Copy
	//
	CopyFilter *poCopyFilter = new CopyFilter;
			
	// set data from config
	CloneData* poCloneData = new CloneData(goVolumeConfig.getData());
	poCloneData->setSrc("/dev/urandom");
	poCloneData->setDest(goVolumeConfig.getData().getSrc());
	
	// set filter data
	poCopyFilter->setSrcData(poCloneData);
	poCopyFilter->setLimit(ullVolumeSize);
	
	// set filter run mode
	if (bUseFullAllocation)
		poCopyFilter->setRunMode(CopyFilter::kRunCopyWithLimitStartNew);
	else
		poCopyFilter->setRunMode(CopyFilter::kRunTruncateWithLimitStartNew);
	
	// set other data
	poCopyFilter->setCurrentSpeed(&gfCurrentSpeed);
	poCopyFilter->setETA(&gstETA);
	poCopyFilter->setPctDone(&giCopyPctDone);
	
	// add filter to pline
	addFilter(poCopyFilter);
	
	// 
	// Set FSys filter
	// 
	Data *poLoopDestData = 0;
	MountData* poMountData = goVolumeConfig.getMountData();
	
	poLoopDestData = 
		(bUseCrypt) ? (Data*)goVolumeConfig.getCryptData() : (Data*)goVolumeConfig.getMountData();
	
	//
	// Loop open
	//
	LoopFilter *poLoopAttachFilter = new LoopFilter;
	
	// set data from config
	LoopData* poLoopData = (LoopData*)goVolumeConfig.getDataPtr();
			
	// set filter data
	poLoopAttachFilter->setSrcData(poLoopData);
	poLoopAttachFilter->setDestData(poLoopDestData);
	
	// set filter run mode
	poLoopAttachFilter->setRunMode(MountBaseFilter::kRunAttach);
	
	// add filter to pline
	addFilter(poLoopAttachFilter);
	
	//
	// Encrypt
	//
	if (bUseCrypt)
	{
		//
		// format
		//
		MkCryptFilter *poMkCryptFilter = new MkCryptFilter;
		
		// set filter data
		poMkCryptFilter->setSrcData((CryptData*)poLoopDestData);
		poMkCryptFilter->setCipher(stCipher);
		poMkCryptFilter->setCBC(stCBC);
		poMkCryptFilter->setKeySize(stKeySize);
		
		// add filter to pline
		addFilter(poMkCryptFilter);
		
		//
		// open
		//
		CryptFilter *poCryptAttachFilter = new CryptFilter;
		
		// set filter data
		poCryptAttachFilter->setSrcData((CryptData*)poLoopDestData);
		poCryptAttachFilter->setDestData(poMountData);
		
		// set filter run mode
		poCryptAttachFilter->setRunMode(MountBaseFilter::kRunAttach);
		
		// add filter to pline
		addFilter(poCryptAttachFilter);
	}
	
	//
	// Create FSys
	//
	MkFSysFilter *poMkFSysFilter = new MkFSysFilter;
	
	// set data from config
	poMkFSysFilter->setSrcData(poMountData);
	
	// add filter to pline
	addFilter(poMkFSysFilter);
	
	//
	// Encrypt close
	//
	if (bUseCrypt)
	{
		gpoCryptDetachFilter = new CryptFilter;
		
		// set filter data
		gpoCryptDetachFilter->setSrcData((CryptData*)poLoopDestData);
		gpoCryptDetachFilter->setDestData(poMountData);
		
		// set filter run mode
		gpoCryptDetachFilter->setRunMode(MountBaseFilter::kRunDetach);
		
		// add filter to pline
		addFilter(gpoCryptDetachFilter);
	}
	
	//
	// Loop close
	//
	gpoLoopDetachFilter = new LoopFilter;
	
	// set filter data
	gpoLoopDetachFilter->setSrcData(poLoopData);
	gpoLoopDetachFilter->setDestData(poLoopDestData);
	
	// set filter run mode
	gpoLoopDetachFilter->setRunMode(MountBaseFilter::kRunDetach);
	
	// add filter to pline
	addFilter(gpoLoopDetachFilter);
}

//==============================================================================

MkDiskImagePline::~MkDiskImagePline()
{
}

//==============================================================================

const Glib::ustring MkDiskImagePline::getInfoMessage()
{
	Glib::ustring stMsg = _("Creating volume...");
	
	if (gbCopying)
	{
		// Copying
		Glib::ustring stSpeed = 
		Glib::ustring::format(std::fixed, std::setprecision(2), gfCurrentSpeed);
			
		stMsg = Util::compose(	_("Creating disk image at %sM/s, %d%% done. ETA %s..."),
								stSpeed.c_str(),
								giCopyPctDone,
								(gstETA.empty()) ? _("inf") : gstETA.c_str());
	
		int iPctDone = ((float)giCopyPctDone/(float)100)*(float)60;
		giPctDone = iPctDone;
	}
	if (gbCryptFormat)
	{
		giPctDone = 65;
		stMsg = _("Encrypting volume...");
	}
	else if (gbCryptOpen)
	{
		giPctDone = 70;
		stMsg = _("Opening secured volume...");
	}
	else if (gbMkFSys)
	{
		giPctDone = 80;
		stMsg = _("Initializing filesystem, please wait...");
	}
		
	return stMsg;
}

//==============================================================================

bool MkDiskImagePline::onFilterPreRun(Filter *poFilter)
{
	gbCopying = (poFilter->getFilterName() == "Copy");
	gbCryptFormat = (poFilter->getFilterName() == "MkCrypt");
	gbCryptOpen = (poFilter->getFilterName() == "Crypt" 
					&& poFilter != gpoCryptDetachFilter);
	gbMkFSys = (poFilter->getFilterName() == "MkFSys");
	
	return ProcessBasePline::onFilterPreRun(poFilter);
}

//==============================================================================

bool MkDiskImagePline::run()
{
	assert(gpoLoopDetachFilter);
	
	// manage completed signal locally
	Pline::skipCompletedSignal(true);
	
	bool bRet = Pline::run();
	
	if (!bRet)
	{
		if (gpoCryptDetachFilter)
			gpoCryptDetachFilter->run();
			
		gpoLoopDetachFilter->run();
	}
	
	m_signal_completed(bRet);
	
	return bRet;
}

//==============================================================================

} // Pline	
