/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MkVolumePline.h"
#include "QDebug.h"
#include "../Util.h"

#include "CryptFilter.h"
#include "MountFilter.h"
#include "MkFSysFilter.h"
#include "MkCryptFilter.h"

#include <iostream>
#include <iomanip>
#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

MkVolumePline::MkVolumePline( 	const eDataMode eDataModeItem,
								VolumeConfig *poVolumeConfig,
								const Glib::ustring& stCipher,
								const Glib::ustring& stCBC,
								const Glib::ustring& stKeySize)
: ProcessBasePline(eDataModeItem, poVolumeConfig)
{
	assert(goVolumeConfig.getCryptData());
	assert(goVolumeConfig.getMountData());
	
	gbCryptFormat = false;
	gbCryptOpen = false;
	gbMkFSys = false;
	
	giPctDone = 0;
	gpoCryptDetachFilter = 0;
	
	//
	// format crypt
	//
	MkCryptFilter *poMkCryptFilter = new MkCryptFilter;
	
	// set filter data
	poMkCryptFilter->setSrcData(goVolumeConfig.getCryptData());
	poMkCryptFilter->setCipher(stCipher);
	poMkCryptFilter->setCBC(stCBC);
	poMkCryptFilter->setKeySize(stKeySize);
	
	// add filter to pline
	addFilter(poMkCryptFilter);
	
	//
	// open
	//
	CryptFilter *poCryptAttachFilter = new CryptFilter;
	
	// set filter data
	poCryptAttachFilter->setSrcData(goVolumeConfig.getCryptData());
	poCryptAttachFilter->setDestData(goVolumeConfig.getMountData());
	
	// set filter run mode
	poCryptAttachFilter->setRunMode(MountBaseFilter::kRunAttach);
	
	// add filter to pline
	addFilter(poCryptAttachFilter);

	//
	// Create FSys
	//
	MkFSysFilter *poMkFSysFilter = new MkFSysFilter;
	
	// set data from config
	poMkFSysFilter->setSrcData(goVolumeConfig.getMountData());
	
	// add filter to pline
	addFilter(poMkFSysFilter);
	
	//
	// Encrypt close
	//
	gpoCryptDetachFilter = new CryptFilter;
		
	// set filter data
	gpoCryptDetachFilter->setSrcData(goVolumeConfig.getCryptData());
	gpoCryptDetachFilter->setDestData(goVolumeConfig.getMountData());
		
	// set filter run mode
	gpoCryptDetachFilter->setRunMode(MountBaseFilter::kRunDetach);
		
	// add filter to pline
	addFilter(gpoCryptDetachFilter);
}

//==============================================================================

MkVolumePline::~MkVolumePline()
{
}

//==============================================================================

const Glib::ustring MkVolumePline::getInfoMessage()
{
	if (gbCryptFormat)
	{
		giPctDone = 0;
		return _("Encrypting volume...");
	}
	else if (gbCryptOpen)
	{
		giPctDone = 10;
		return _("Opening secured volume...");
	}
	else if (gbMkFSys)
	{
		giPctDone = 15;
		return _("Initializing filesystem, please wait...");
	}
	else	
	{
		giPctDone = 90;
		return _("Completing...");
	}
}

//==============================================================================

bool MkVolumePline::onFilterPreRun(Filter *poFilter)
{
	gbCryptFormat = (poFilter->getFilterName() == "MkCrypt");
	gbCryptOpen = (poFilter->getFilterName() == "Crypt" 
					&& poFilter != gpoCryptDetachFilter);
	gbMkFSys = (poFilter->getFilterName() == "MkFSys");
	
	return ProcessBasePline::onFilterPreRun(poFilter);
}

//==============================================================================

bool MkVolumePline::run()
{
	assert(gpoCryptDetachFilter);
	
	// manage completed signal locally
	Pline::skipCompletedSignal(true);
	
	bool bRet = Pline::run();
	
	if (!bRet)
		gpoCryptDetachFilter->run();
	
	m_signal_completed(bRet);
	
	return bRet;
}

//==============================================================================

} // Pline	
