/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MOUNTUTIL_H
#define MOUNTUTIL_H

#include <gtkmm.h>

namespace Pline
{

class MountUtil
{
public:
	MountUtil();
	~MountUtil();

	static bool getDiskUsage(   const Glib::ustring& stDir,
								Glib::ustring *pstDiskSize,
								Glib::ustring *pstSpaceUsed,
								Glib::ustring *pstSpaceAvailable,
								int *piDiskUsagePct,
								Glib::ustring *pstErrMsg = 0);		
								
	static bool getFSys(const Glib::ustring& stSrc, 
						Glib::ustring *pstFSys,
						Glib::ustring *pstErrMsg = 0);
	
	static bool isMounted(  const Glib::ustring& stSrc,
							const Glib::ustring& stDir);
	static bool isMountedDir(const Glib::ustring& stDir);
	static bool isMountedSrc(const Glib::ustring& stSrc);
	static bool getMountPoint(	const Glib::ustring& stSrc,
								Glib::ustring *pstDir);
	
	static bool mount(  const Glib::ustring& stSrc, 
						const Glib::ustring& stDir,
						const Glib::ustring& stFSys, 
						unsigned long ulMountFlags,
						Glib::ustring *pstErrMsg = 0);

	static bool umount( const Glib::ustring& stDir,
						Glib::ustring *pstErrMsg = 0);
	
	// kernel mods
	static bool getAvailFSysMod(std::vector<Glib::ustring> *pvecMod);
	static const Glib::ustring getFSysModPath();
	
	// filesystem
	static bool getFSysList(std::vector<Glib::ustring> *pvecFSys);
	
	// validate
	static bool isValidLabel(   const Glib::ustring& stLabel, 
								Glib::ustring *pstErrMsg = 0);
	static bool isValidSrc( const Glib::ustring& stSrc, 
							Glib::ustring *pstErrMsg = 0);
	static bool isValidDir( const Glib::ustring& stDir, 
							Glib::ustring *pstErrMsg = 0);
	static bool isValidFSys(const Glib::ustring& stFSys, 
							Glib::ustring *pstErrMsg = 0);
	
protected:

private:
	static bool getFSysListMountable(std::vector<Glib::ustring> *pvecFSys);
	
};

} // Pline		
#endif /* MOUNTUTIL_H */
