/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLINE_H
#define PLINE_H

#include "Data.h"
#include "../VolumeConfig.h"

#include <gtkmm.h>
#include <vector>

namespace Pline
{

typedef enum _DATAMODE_
{
	kNoSignalDataMode,
	kWithSignalDataMode,
}
eDataMode;
	
class Filter;

class Pline
{
public:
	Pline(  const eDataMode eDataModeItem,
			VolumeConfig *poVolumeConfig);
	virtual ~Pline();

	// this method allows the filter to prompt data to the the user
	// with signal data mode this method must be called with signal_req_data
	virtual void askData();
		
	// run
	bool isRunning();
	virtual bool run();

	// errors
	typedef struct _ERROR_
	{
		Glib::ustring   stErrMsg;
		Glib::ustring   stFilterName;
		bool			bIsFilterError;
		bool			bHasCancelledDataReq;
		bool 			bHasCancelledAction;
	}
	tError;
	
	tError getLastError();
		
	// signals
	inline sigc::signal<void>& signal_req_data() {return m_signal_req_data;}
	
	inline sigc::signal<void, bool>& signal_completed() {return m_signal_completed;}
	inline sigc::signal<void>& signal_update() {return m_signal_update;}
			
protected:
	VolumeConfig& goVolumeConfig;
	tError gtLastError;
	
	sigc::signal<void, bool> m_signal_completed;
	inline void skipCompletedSignal(const bool bSkip) {gbSkipCompletedSignal = bSkip;}
	
	void addFilter(Filter *poFilter);
	
	std::vector<Filter*> getFilters();
	void removeFilters();
	void reverseFilters();
	
	virtual void initError();
	
	virtual void onReqData(Filter *poFilter);
	virtual void onUpdate(Filter *poFilter);
	
	virtual bool onFilterPreRun(Filter *poFilter);
	virtual bool onFilterRun(Filter *poFilter, bool bHasRunWithErrors);
	
	// Data signal
	sigc::signal<void>  m_signal_req_data;
	Filter				*gpoReqFilter;
	
private:
	eDataMode	geDataMode; // Data signal
	bool		gbRun; // Running status
	bool 		gbSkipCompletedSignal;
	
	// signals
	sigc::signal<void> m_signal_update;
	sigc::connection goConfigChangedFun;
		
	// Running status
	Glib::Mutex gmtxRun;
	
	// Filters
	std::vector<Filter*> gvecFilter;
};

} // Pline	
#endif /* PLINE_H */
