/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QCONFIG_H
#define QCONFIG_H

#include <gtkmm.h>
#include <list>

#include "VolumeConfig.h"

class QConfig
{
public:
	QConfig();
	~QConfig();

	static QConfig& getInstance();
	
	// Main
	bool getSortBy(int *pi);
	
	bool isDisableAutomount();
	bool isHideToolTips();
	bool isSaveWinPos();
	bool isSaveWinSize();
	
	bool getMainWinX(int *pi);
	bool getMainWinY(int *pi);
	bool getMainWinW(int *pi);
	bool getMainWinH(int *pi);
	
	void setSortBy(const int i);
	
	void setDisableAutomount(const bool bDisable);
	void setHideToolTips(const bool bHide);
	void setSaveWinPos(const bool bSave);
	void setSaveWinSize(const bool bSave);
	
	void setMainWinX(const int i);
	void setMainWinY(const int i);
	void setMainWinW(const int i);
	void setMainWinH(const int i);
	
	// Default VolumeConfig lookup dir
	inline const Glib::ustring& getDfCloneDir() {return gstDfCloneDir;}
	inline const Glib::ustring& getDfDir() {return gstDfDir;}
	inline const Glib::ustring& getDfFSys() {return gstDfFSys;}
	inline const Glib::ustring& getDfKey() {return gstDfKey;}
	inline const Glib::ustring& getDfSrc() {return gstDfSrc;}
	
	inline void setDfCloneDir(const Glib::ustring& st) {gstDfCloneDir = st;}
	inline void setDfDir(const Glib::ustring& st) {gstDfDir = st;}
	inline void setDfFSys(const Glib::ustring& st) {gstDfFSys = st;}
	inline void setDfKey(const Glib::ustring& st) {gstDfKey = st;}
	inline void setDfSrc(const Glib::ustring& st) {gstDfSrc = st;}
	
	// VolumeConfig
	bool addVolume(VolumeConfig* poVolumeConfig);
	void removeVolume(VolumeConfig* poVolumeConfig);
	
	bool findVolume(const Glib::ustring& stLabel,
					VolumeConfig& oVolumeConfig);

	std::list<VolumeConfig*>& getVolumes() {return glstVolumeConfig;}	
	
protected:

private:
	bool gbSaveBackup;
	Glib::ustring gstConfigFilename;
	
	Glib::KeyFile goKeyFile;
	std::list<VolumeConfig*> glstVolumeConfig;
	Glib::ustring gstDfDir;
	Glib::ustring gstDfFSys;
	Glib::ustring gstDfSrc;
	Glib::ustring gstDfKey;
	Glib::ustring gstDfCloneDir;
	
	bool loadConfig();
	Glib::ustring loadDefaultConfig() const;
	bool saveConfig();
};
	
#endif /* QCONFIG_H */
