/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "QDebug.h"
#include <string>
#include <iostream>

//==============================================================================

#ifdef QDEBUG
	static int debug_flags = DEBUG_ALL;
#else
	static int debug_flags = NO_DEBUG;
#endif

//==============================================================================

void __debug_init(int flags)
{
	debug_flags = flags;
}

//==============================================================================

bool debug_check_flags(int flag)
{
	if(G_UNLIKELY(debug_flags & DEBUG_ALL))
		return true;

	return G_UNLIKELY(debug_flags & flag);
}

//==============================================================================

void __debug(   int flag, 
				const gchar* file, 
				const gint line, 
				const gchar* fonction)
{
	if(G_UNLIKELY(debug_flags & flag) || G_UNLIKELY(debug_flags & DEBUG_ALL))
	{
		g_print("%s:%d (%s)\n", file, line, fonction);
		fflush(stdout);
	}
}

//==============================================================================

void __debug_message(   int flag, 
						const gchar* file, 
						gint line, 
						const gchar* fonction, 
						const char *format, ...)
{
	if (G_UNLIKELY(debug_flags & flag) || G_UNLIKELY(debug_flags & DEBUG_ALL))
	{
		va_list args;
		gchar *msg = NULL;

		g_return_if_fail(format);

		va_start(args, format);
		msg = g_strdup_vprintf(format, args);
		va_end(args);

		gchar *file_basename = g_path_get_basename(file);

		g_print("%s:%s(%d) %s\n", file_basename, fonction, line, msg);

		fflush(stdout);

		g_free(file_basename);
		g_free(msg);
	}
}

//==============================================================================
