/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "QDialog.h"
#include "QDebug.h"
#include "Icon.h"
#include "eMountWin.h"

//==============================================================================

QDialog::QDialog(const tConfig& tConfigItem)
: Gtk::Dialog()
{
	gbWaitData = false;
	
	(tConfigItem.poParent) ? 
		set_transient_for(*tConfigItem.poParent) : eMountWin::setChild(*this);

	set_title(tConfigItem.stTitle);
	
	set_resizable(true);
	set_has_separator(false);
	
	//
	// logo and extra hbox
	//
	Gtk::HBox* hbox = manage(new Gtk::HBox(false, 12));
	hbox->set_border_width(12);
	
	get_vbox()->pack_start(*hbox, false, false);

	if (Icon::getInstance().drawSysIcon32())
	{
		Gtk::Image* img = manage(new Gtk::Image(*Icon::getInstance().drawSysIcon32()));
		img->set_alignment(0.0, 0.0);
		hbox->pack_start(*img, false, false);
	}
	
	// vbox used by derived
	gpoVBox = manage(new Gtk::VBox(false, 12));
	hbox->pack_start(*gpoVBox);
	
	// min width vbox
	Gtk::VBox* vbox2 = manage(new Gtk::VBox(false, 3));
	vbox2->set_size_request(tConfigItem.iWidth, -1);
	gpoVBox->pack_start(*vbox2);
	
	if (tConfigItem.bWaitData)
	{
		gbWaitData = true;

		eMountWin::setUserIdle(false);
	}
}

//==============================================================================

QDialog::QDialog(BaseObjectType* cobject)
: Gtk::Dialog(cobject)
{
	eMountWin::setChild(*this);

	gbWaitData = false;
}

//==============================================================================

QDialog::~QDialog()
{
	if (gbWaitData)
		eMountWin::setUserIdle(true);
}

//==============================================================================

void QDialog::initConfig(tConfig& tConfigItem)
{
	tConfigItem.poParent = 0;
	tConfigItem.iWidth = 320; // min width
	tConfigItem.bWaitData = true;
	tConfigItem.stTitle.clear();
}

//==============================================================================

void QDialog::on_response(int response_id)
{
	hide();
}

//==============================================================================
