/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "VolumeCellRenderer.h"
#include "QDebug.h"
#include "VolumeTreeView.h"
#include "Icon.h"

#include <algorithm>
#include <limits.h>
#include <assert.h>

VolumeCellRenderer::VolumeCellRenderer()
: Glib::ObjectBase(typeid(VolumeCellRenderer)),
  Gtk::CellRenderer(),
  gdScreenResolution(Gdk::Screen::get_default()->get_resolution()),
  m_property_index(*this, "index", 0),
  m_property_pos(*this, "pos", -1),
  m_property_btn_cancel(*this, "btn_cancel", 0),
  m_property_btn_eject(*this, "btn_eject", 0),
  m_property_btn_mount(*this, "btn_mount", 0),
  m_property_draw_info(*this, "info", ""),
  m_property_draw_label(*this, "label", ""),
  m_property_draw_progress_full(*this, "progress_full", false),
  m_property_draw_progress_pct_gauge(*this, "progress_pct_gauge", -1),
  m_property_draw_progress_pct_used(*this, "progress_pct_used", -1)
{
	property_mode() = Gtk::CELL_RENDERER_MODE_ACTIVATABLE;	
}

//==============================================================================

VolumeCellRenderer::~VolumeCellRenderer()
{
}

//==============================================================================

bool VolumeCellRenderer::activate_vfunc(GdkEvent* event, 
										Gtk::Widget& widget, 
										const Glib::ustring& path, 
										const Gdk::Rectangle& background_area, 
										const Gdk::Rectangle& cell_area, 
										Gtk::CellRendererState flags)
{
	if (!event)
		return false;
		
	m_signal_activate(  property_index(),
						((GdkEventButton*)event)->x,
						((GdkEventButton*)event)->y);
	
	return true;
}

//==============================================================================

int VolumeCellRenderer::calcRowHeight(  const Glib::ustring& stDrawInfo,
										const Glib::ustring& stDrawLabel,
										const bool bDrawProgress)
{
	int iy = 0;
	
	// Info
	if (!stDrawInfo.empty())
	{
		iy += 5;
		
		int iFontSize = 7;											
		iy += ceil(iFontSize*(gdScreenResolution/72));
	}
		
	if (!stDrawLabel.empty())
	{
		iy += 11;

		int iFontSize = 8;
		iy += ceil(iFontSize*(gdScreenResolution/72));
	}
	
	// Progress
	if (bDrawProgress)
	{
	
		iy += 7;
		iy += 17;
	}
	
	// Separator
	iy += 11;	

	return iy;
}
		
//==============================================================================

int VolumeCellRenderer::createNewRowIndex()
{
	Glib::Rand		oRand;
	tLayout			tLayoutNewItem;
	int				ci = 0;
	
	// find new random index
	while (true)
	{
		if (ci >= INT_MAX)
			return 0;
		
		tLayoutNewItem.iIndex = oRand.get_int_range(1, INT_MAX);
		
		std::list<tLayout>::iterator ite =
				std::find(glstLayout.begin(), glstLayout.end(), tLayoutNewItem);
		
		if (ite == glstLayout.end())
		{
			glstLayout.push_back(tLayoutNewItem);
			
			return tLayoutNewItem.iIndex;
		}
			
		ci++;
	}
	
	return 0;
}

//==============================================================================

void VolumeCellRenderer::drawProgressBkg(	Cairo::RefPtr<Cairo::Context>& cr, 
											const Gdk::Rectangle& rectangle,
											const int iHeight,
											Gdk::Rectangle *poInnerRect)
{
	Gdk::Color start_color;
	Gdk::Color end_color;

	double x0 = rectangle.get_x();
	double y0 = rectangle.get_y();
	double x1 = rectangle.get_width();
	double y1 = rectangle.get_height();

	cr->save();

	// background layer 1
	start_color.parse("#323232");
	end_color.parse("#5A5A5A");
		
	Cairo::RefPtr< Cairo::LinearGradient > refBackground_gradient = Cairo::LinearGradient::create (
		x0, y0, x0, y0+y1+iHeight);
    
	// Set grandient colors
	refBackground_gradient->add_color_stop_rgb (0, 
		start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
	refBackground_gradient->add_color_stop_rgb (0.8, 
		end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

	cr->set_source(refBackground_gradient);
	cr->rectangle(x0, y0, x1, y1);
	cr->fill();
    
    // background layer 2
	start_color.parse("#282828");
	end_color.parse("#AAAAAA");
		
	refBackground_gradient = Cairo::LinearGradient::create (
		x0, y0, x0, y0+y1+iHeight);
    
	// Set grandient colors
	refBackground_gradient->add_color_stop_rgb (0, 
		start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
	refBackground_gradient->add_color_stop_rgb (0.8, 
		end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

	cr->set_source(refBackground_gradient);
	cr->rectangle(x0+1, y0+1, x1-2, y1-2);
	cr->fill();
	
	// draw vertical lines
	double dHSpace = x1 / (double)10;

	for (double dx=x0+dHSpace; dx<(x0+x1)-(dHSpace-1); dx+=dHSpace)
	{
		start_color.parse("#050505");
		end_color.parse("#707070");
		
		refBackground_gradient = Cairo::LinearGradient::create (
			dx, y0, dx, y0+y1+iHeight);
    
		// Set grandient colors
		refBackground_gradient->add_color_stop_rgb (0, 
			start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
		refBackground_gradient->add_color_stop_rgb (1, 
			end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

		cr->set_source(refBackground_gradient);
		
		cr->rectangle(dx, y0+2, 1, y1-4);
		cr->fill();
	}
	
	cr->restore();
	refBackground_gradient.clear();
	
	*poInnerRect = Gdk::Rectangle(x0+2, y0+2, x1-4, y1-4);
}

//==============================================================================

void VolumeCellRenderer::drawProgressFull(  Cairo::RefPtr<Cairo::Context>& cr, 
											const Gdk::Rectangle& rectangle,
											const int iHeight)
{
	Gdk::Color start_color;
	Gdk::Color end_color;

	double x0 = rectangle.get_x();
	double y0 = rectangle.get_y();
	double x1 = rectangle.get_width();
	double y1 = rectangle.get_height();

	cr->save();

	//start_color.parse("#FFC55E");
	start_color.parse("#FFE859");
	end_color.parse("#FFFFFF");
		
	Cairo::RefPtr< Cairo::LinearGradient > refBackground_gradient = Cairo::LinearGradient::create (
		x0, y0, x0, y0+y1+iHeight);
    
	// Set grandient colors
	refBackground_gradient->add_color_stop_rgb (0, 
		start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
	refBackground_gradient->add_color_stop_rgb (0.5, 
		end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

	cr->set_source(refBackground_gradient);
	cr->rectangle(x0, y0, x1, y1);
	cr->fill();
	
	cr->restore();
	refBackground_gradient.clear();
}
							
//==============================================================================

void VolumeCellRenderer::drawProgressGauge( Cairo::RefPtr<Cairo::Context>& cr, 
											const Gdk::Rectangle& rectangle,
											const int iPct)
{
	Gdk::Color start_color;
	Gdk::Color end_color;

	double x0 = rectangle.get_x();
	double y0 = rectangle.get_y();
	double x1 = rectangle.get_width();
	double y1 = rectangle.get_height();

	int iPct2 = (iPct > 100 || iPct < 0) ? 100 : iPct;
	
	cr->save();

	// background layer 1
	start_color.parse("#3C4AFF");
	end_color.parse("#FFFFFF");
		
	Cairo::RefPtr< Cairo::LinearGradient > refBackground_gradient = Cairo::LinearGradient::create (
		x0, y0, x0, y0+y1);
    
	// Set grandient colors
	refBackground_gradient->add_color_stop_rgb (0, 
		start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
	refBackground_gradient->add_color_stop_rgb (1, 
		end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

	double dEndAt = (x1 * (100-(double)iPct2))/100;
	
	cr->set_source(refBackground_gradient);
	cr->rectangle(x0, y0, x1-dEndAt, y1);
	cr->fill();
	
	cr->restore();
	refBackground_gradient.clear();
}

//==============================================================================

void VolumeCellRenderer::drawProgressUsedSpace( Cairo::RefPtr<Cairo::Context>& cr, 
												const Gdk::Rectangle& rectangle,
												const int iPctUsed)
{
	Gdk::Color start_color;
	Gdk::Color mid_color;
	Gdk::Color end_color;

	double x0 = rectangle.get_x();
	double y0 = rectangle.get_y();
	double x1 = rectangle.get_width();
	double y1 = rectangle.get_height();

	int iPct = (iPctUsed > 100 || iPctUsed < 0) ? 100 : iPctUsed;

	cr->save();

	start_color.parse("#73FF73");
	mid_color.parse("#3F3FED");
	end_color.parse("#E60000");
		
	Cairo::RefPtr< Cairo::LinearGradient > refBackground_gradient = Cairo::LinearGradient::create (
		x0, y0, x1, y0);
    
	// Set grandient colors
	refBackground_gradient->add_color_stop_rgb (0, 
		start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
	refBackground_gradient->add_color_stop_rgb (0.8, 
		mid_color.get_red_p(), mid_color.get_green_p(), mid_color.get_blue_p());

	refBackground_gradient->add_color_stop_rgb (1, 
		end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());
			
	double dEndAt = (x1 * (100-(double)iPct))/100;
	
	cr->set_source(refBackground_gradient);
	cr->rectangle(x0, y0, x1-dEndAt, y1);
	cr->fill();
	
	cr->restore();
	refBackground_gradient.clear();
}
																						
//==============================================================================

bool VolumeCellRenderer::isControl( const int ix, 
									const int iy,
									const int iRowIndexStart,
									const int iRowIndexEnd,
									tLayoutCtrl *ptLayoutCtrl)
{
	bool bCheckRect = false;
	
	for (std::list<tLayout>::iterator ite = glstLayout.begin();
		ite != glstLayout.end();
		++ite)
	{	
		tLayout *ptLayout = &(*ite);
				
		if (ptLayout->iIndex == iRowIndexStart)
			bCheckRect = true;
			
		if (bCheckRect)
		{	
			if (isCursorInRect(ix, iy, ptLayout->oBtnCancelRect))
			{
				ptLayoutCtrl->iIndex = ptLayout->iIndex;
				ptLayoutCtrl->eBtnStateItem = ptLayout->eBtnCancelState;
				ptLayoutCtrl->eLayoutCtrlTypeItem = kBtnCancel;
			
				return true;
			}
			else if (isCursorInRect(ix, iy, ptLayout->oBtnEjectRect))
			{
				ptLayoutCtrl->iIndex = ptLayout->iIndex;
				ptLayoutCtrl->eBtnStateItem = ptLayout->eBtnEjectState;
				ptLayoutCtrl->eLayoutCtrlTypeItem = kBtnEject;
			
				return true;
			}
			else if (isCursorInRect(ix, iy, ptLayout->oBtnMountRect))
			{
				ptLayoutCtrl->iIndex = ptLayout->iIndex;
				ptLayoutCtrl->eBtnStateItem = ptLayout->eBtnMountState;
				ptLayoutCtrl->eLayoutCtrlTypeItem = kBtnMount;
			
				return true;
			}
		}
			
		if (ptLayout->iIndex == iRowIndexEnd)
			bCheckRect = false;
	}
	
	return false;
}

//==============================================================================

bool VolumeCellRenderer::isProgressVisible()
{
	if (property_draw_progress_full()
		|| property_draw_progress_pct_gauge() >= 0
		|| property_draw_progress_pct_used() >= 0)
	{
		return true;
	}
	
	return false;
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_index()
{
	return m_property_index.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_pos()
{
	return m_property_pos.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_btn_cancel()
{
	return m_property_btn_cancel.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_btn_eject()
{
	return m_property_btn_eject.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_btn_mount()
{
	return m_property_btn_mount.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<Glib::ustring> VolumeCellRenderer::property_draw_info()
{
	return m_property_draw_info.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<Glib::ustring> VolumeCellRenderer::property_draw_label()
{
	return m_property_draw_label.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<bool> VolumeCellRenderer::property_draw_progress_full()
{
	return m_property_draw_progress_full.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_draw_progress_pct_gauge()
{
	return m_property_draw_progress_pct_gauge.get_proxy();
}

//==============================================================================

Glib::PropertyProxy<int> VolumeCellRenderer::property_draw_progress_pct_used()
{
	return m_property_draw_progress_pct_used.get_proxy();
}

//==============================================================================

bool VolumeCellRenderer::removeRowIndex(const int iIndex)
{
	tLayout tLayoutEraseItem;
	
	tLayoutEraseItem.iIndex = iIndex;
	
	std::list<tLayout>::iterator ite =
			std::find(glstLayout.begin(), glstLayout.end(), tLayoutEraseItem);
		
	if (ite != glstLayout.end())
	{
		glstLayout.remove(*ite);
			
		return true;
	}
	
	return false;
}

//==============================================================================

void VolumeCellRenderer::render_vfunc(  const Glib::RefPtr<Gdk::Drawable>& window, 
										Gtk::Widget& widget, 
										const Gdk::Rectangle& background_area, 
										const Gdk::Rectangle& cell_area, 
										const Gdk::Rectangle& expose_area, 
										Gtk::CellRendererState flags)
{
	// Draw layout
	bool bDrawLayout = false; 
	Gdk::Color oLayoutColor;
	
	oLayoutColor.parse("#FF0000");
	
#ifdef QDEBUG
	bDrawLayout = false;
#endif

	// Get cell size
    int x_offset = 0, y_offset = 0, width = 0, height = 0;
    get_size(widget, cell_area, x_offset, y_offset, width, height);
    
    // Create the graphic context
    Glib::RefPtr<Gdk::GC> gc = Gdk::GC::create(window);
    
    // Get Gdk window
	Glib::RefPtr<Gdk::Window> win = Glib::RefPtr<Gdk::Window>::cast_dynamic(window);
	
	// Create Cairo context
	Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
	
    // Set state var
	Gtk::StateType state;
	
	if ((flags & Gtk::CELL_RENDERER_SELECTED) != 0)
		state = Gtk::STATE_SELECTED;
	else
		state = Gtk::STATE_NORMAL;

	//
	// Draw
	//
	
	// Set properties var
	eBtnState eBtnCancelState =
		(eBtnState)((int)property_btn_cancel());
	eBtnState eBtnEjectState = 
		(eBtnState)((int)property_btn_eject());
	eBtnState eBtnMountState = 
		(eBtnState)((int)property_btn_mount());
	
	Glib::ustring stInfo		= property_draw_info();
	Glib::ustring stLabel		= property_draw_label();
	bool bProgressFull			= property_draw_progress_full();
	int iProgressPctGauge	    = property_draw_progress_pct_gauge();
	int iProgressPctUsed	    = property_draw_progress_pct_used();
	
	tLayout tLayoutItem;
	tLayoutItem.iIndex = property_index();
	
	// Current pos
	int iy = 0;
	int ixLPad = 5;
	 
	// Background
	if (state != Gtk::STATE_SELECTED)
	{
		Gdk::Color start_color;
		Gdk::Color end_color;

		double x0 = background_area.get_x();
		double y0 = background_area.get_y();
		double x1 = background_area.get_width();
		double y1 = background_area.get_height();

		double height2 = y0 + y_offset;
				
		start_color.parse("#FFFFFF");
		//start_color.parse("#000000"); // testing
		end_color.parse("#CFCFCF");
		
		Cairo::RefPtr< Cairo::LinearGradient > refBackground_gradient = Cairo::LinearGradient::create (
			x0, y0, x0, y1+height2);
    
		// Set grandient colors
		refBackground_gradient->add_color_stop_rgb (0, 
			start_color.get_red_p(), start_color.get_green_p(), start_color.get_blue_p());
    
		refBackground_gradient->add_color_stop_rgb (1, 
			end_color.get_red_p(), end_color.get_green_p(), end_color.get_blue_p());

		cr->save();
		cr->set_source(refBackground_gradient);
		cr->rectangle(x0, y0, x1, y1);
		cr->fill();
		cr->restore();
    
		refBackground_gradient.clear();
	}
                                                       
	// Volume label
	if (!stLabel.empty())
	{
		iy += 11;
		
		// create layout with label value
		Glib::RefPtr<Pango::Layout> ptrLabelLayout = widget.create_pango_layout(stLabel); 
		
		// set font
		Pango::FontDescription oFont;
		int iFontSize = 8;
		oFont.set_weight(Pango::WEIGHT_BOLD);
		oFont.set_size(iFontSize*Pango::SCALE);
		ptrLabelLayout->set_font_description(oFont);

		widget.get_style()->paint_layout(   win,
											state,
											true,
											cell_area,
											widget,
											"",
											cell_area.get_x() + x_offset + ixLPad,
											cell_area.get_y() + y_offset + iy,
											ptrLabelLayout);
											
		iy += ceil(iFontSize*(gdScreenResolution/72));
	}
	
	// Progress
	if (isProgressVisible())
	{
		iy += 7;

		double x0 = cell_area.get_x() + x_offset + ixLPad;
		double y0 = cell_area.get_y() + y_offset + iy;
		double x1 = cell_area.get_width() - 65;
		double y1 = 11;

		if (eBtnCancelState != kBtnHidden)
			x1 += 20;
			
		Gdk::Rectangle oProgressRect(x0, y0, x1, y1);
		Gdk::Rectangle oProgressInnerRect;
		
		drawProgressBkg(cr, oProgressRect, y_offset, &oProgressInnerRect);
		
		if (bProgressFull)
			drawProgressFull(cr, oProgressInnerRect, y_offset);
		else if (iProgressPctGauge >= 0)
			drawProgressGauge(cr, oProgressInnerRect, iProgressPctGauge);
		else if (iProgressPctUsed >= 0)
			drawProgressUsedSpace(cr, oProgressInnerRect, iProgressPctUsed);			
		
		iy += 17;
	}
	
	// Info
	if (!stInfo.empty())
	{
		iy += 5;
		
		Glib::RefPtr<Pango::Layout> ptrInfoLayout = widget.create_pango_layout(stInfo); 
		
		// set font
		Pango::FontDescription oFont;
		int iFontSize = 7;
		oFont.set_weight(Pango::WEIGHT_NORMAL);
		oFont.set_size(iFontSize*Pango::SCALE);
		ptrInfoLayout->set_font_description(oFont);
		
		widget.get_style()->paint_layout(   win,
											Gtk::STATE_NORMAL,
											true,
											cell_area,
											widget,
											"",
											cell_area.get_x() + x_offset + ixLPad,
											cell_area.get_y() + y_offset + iy,
											ptrInfoLayout);
											
		iy += ceil(iFontSize*(gdScreenResolution/72));
	}
										
	// Separator
	Gdk::Color oSepColor;
	
	iy += 11;
		
	if (oSepColor.parse("#797979"))
		gc->set_rgb_fg_color(oSepColor);
			
	window->draw_rectangle( gc,
							true,
							background_area.get_x(),
							background_area.get_y()+background_area.get_height()-1,
							background_area.get_width(),
                            1);
              
	//              
	// Buttons
	//
	
	// Mount button
	if (eBtnMountState != kBtnHidden)
	{
		bool bIconOK = false;
		Glib::RefPtr<Gdk::Pixbuf> oBtnIcon;
		
		if (eBtnMountState == kBtnDisabled)
		{
			if (Icon::getInstance().drawBtnMountDisabledIcon())
			{
				oBtnIcon = *Icon::getInstance().drawBtnMountDisabledIcon();
				bIconOK = true;
			}
		}
		else if (eBtnMountState == kBtnNormal)
		{
			if (Icon::getInstance().drawBtnMountNormalIcon())
			{
				oBtnIcon = *Icon::getInstance().drawBtnMountNormalIcon();
				bIconOK = true;
			}
		}
		
		if (bIconOK)
		{
			int iPicWidth = oBtnIcon->get_width();
			int iyOffset = 0, ixOffset = 0;
	
			if (isProgressVisible())
			{
				ixOffset = cell_area.get_width() - 56;
				iyOffset = 23;
			}
			else
			{
				ixOffset = cell_area.get_width() - (iPicWidth + ixLPad);
				iyOffset = 6;
			}
			
			tLayoutItem.eBtnMountState = eBtnMountState;
			
			tLayoutItem.oBtnMountRect = Gdk::Rectangle( cell_area.get_x() + x_offset + ixOffset,
														cell_area.get_y() + y_offset + iyOffset,
														iPicWidth - 1, iPicWidth - 1);
														
			if (bDrawLayout)
			{
				gc->set_rgb_fg_color(oLayoutColor);
				
				window->draw_rectangle( gc,
										false,
										tLayoutItem.oBtnMountRect.get_x(),
										tLayoutItem.oBtnMountRect.get_y(),
										tLayoutItem.oBtnMountRect.get_width(),
		    		                    tLayoutItem.oBtnMountRect.get_height());
			}

			window->draw_pixbuf(gc,
								oBtnIcon,
								0,
								0,
								cell_area.get_x() + x_offset + ixOffset,
								cell_area.get_y() + y_offset + iyOffset,
								-1,
								-1,
								Gdk::RGB_DITHER_NONE,
								0,
								0);
		}
	}
	
	// Eject button
	if (eBtnEjectState != kBtnHidden)
	{
		if (Icon::getInstance().drawBtnEjectNormalIcon())
		{
			int iPicWidth = (*Icon::getInstance().drawBtnEjectNormalIcon())->get_width();
			int iyOffset = 0, ixOffset = 0;
		
			if (isProgressVisible())
			{
				ixOffset = cell_area.get_width() - (iPicWidth + ixLPad);
				iyOffset = 23;
			}
			else
			{
				ixOffset = cell_area.get_width() - (iPicWidth + ixLPad);
				iyOffset = 6;
			}
			
			tLayoutItem.eBtnEjectState = eBtnEjectState;
			tLayoutItem.oBtnEjectRect = Gdk::Rectangle( cell_area.get_x() + x_offset + ixOffset,
														cell_area.get_y() + y_offset + iyOffset,
														iPicWidth - 1, iPicWidth - 1);
			
			if (bDrawLayout)
			{
				gc->set_rgb_fg_color(oLayoutColor);
				
				window->draw_rectangle( gc,
										false,
										tLayoutItem.oBtnEjectRect.get_x(),
										tLayoutItem.oBtnEjectRect.get_y(),
										tLayoutItem.oBtnEjectRect.get_width(),
		    		                    tLayoutItem.oBtnEjectRect.get_height());
			}
					
			window->draw_pixbuf(gc,
								*Icon::getInstance().drawBtnEjectNormalIcon(),
								0,
								0,
								cell_area.get_x() + x_offset + ixOffset,
								cell_area.get_y() + y_offset + iyOffset,
								-1,
								-1,
								Gdk::RGB_DITHER_NONE,
								0,
								0);
		}
		
		
	}

	// Cancel button
	if (eBtnCancelState != kBtnHidden)
	{
		bool bIconOK = false;
		Glib::RefPtr<Gdk::Pixbuf> oBtnIcon;
		
		if (eBtnCancelState == kBtnDisabled)
		{
			if (Icon::getInstance().drawBtnCancelDisabledIcon())
			{
				oBtnIcon = *Icon::getInstance().drawBtnCancelDisabledIcon();
				bIconOK = true;
			}
		}
		else if (eBtnCancelState == kBtnNormal)
		{
			if (Icon::getInstance().drawBtnCancelNormalIcon())
			{
				oBtnIcon = *Icon::getInstance().drawBtnCancelNormalIcon();
				bIconOK = true;
			}
		}
		
		if (bIconOK)
		{
			int iPicWidth = oBtnIcon->get_width();
			int iyOffset = 0, ixOffset = 0;
	
			ixOffset = cell_area.get_width() - (iPicWidth + ixLPad);
			iyOffset = 23;
			
			tLayoutItem.eBtnCancelState = eBtnCancelState;
			tLayoutItem.oBtnCancelRect = Gdk::Rectangle(cell_area.get_x() + x_offset + ixOffset,
														cell_area.get_y() + y_offset + iyOffset,
														iPicWidth - 1, iPicWidth - 1);
			
			if (bDrawLayout)
			{
				gc->set_rgb_fg_color(oLayoutColor);
				
				window->draw_rectangle( gc,
										false,
										tLayoutItem.oBtnCancelRect.get_x(),
										tLayoutItem.oBtnCancelRect.get_y(),
										tLayoutItem.oBtnCancelRect.get_width(),
		    		                    tLayoutItem.oBtnCancelRect.get_height());
			}
			
			window->draw_pixbuf(gc,
								oBtnIcon,
								0,
								0,
								cell_area.get_x() + x_offset + ixOffset,
								cell_area.get_y() + y_offset + iyOffset,
								-1,
								-1,
								Gdk::RGB_DITHER_NONE,
								0,
								0);
		}
	}
	
	// update layout
	updateLayout(&tLayoutItem);
	
	// Call parent
	Gtk::CellRenderer::render_vfunc(window, widget, background_area, cell_area, expose_area, flags);
}

//==============================================================================

void VolumeCellRenderer::setIndexOrder(const int iIndex, const int iPos)
{
	int xiFountAt = 0;
	
	if (iIndex == 0 || glstLayout.size() <= 1)
		return;
	
	// find element	
	std::list<tLayout>::iterator ite_found;
	
	for (ite_found = glstLayout.begin();
		ite_found != glstLayout.end();
		++ite_found)
	{
		if (((tLayout)*ite_found).iIndex == iIndex)
			break;
			
		xiFountAt++;
	}
	
	if (ite_found == glstLayout.end())
		return;
	
	// find insert point
	std::list<tLayout>::iterator ite = glstLayout.begin();
	
	for (int ci=0; ci<iPos; ++ci)
		++ite;
	
	if (xiFountAt < iPos && ite != glstLayout.end())	
		++ite;
	
	// splice	
	glstLayout.splice(ite, glstLayout, ite_found);
}

//==============================================================================

void VolumeCellRenderer::updateLayout(tLayout *ptLayout)
{
	tLayout tLayoutUpdateItem;
	
	tLayoutUpdateItem.iIndex = ptLayout->iIndex;
		
	std::list<tLayout>::iterator ite =
			std::find(glstLayout.begin(), glstLayout.end(), tLayoutUpdateItem);
		
	if (ite != glstLayout.end())
		*ite = *ptLayout;
}

//==============================================================================
