/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VOLUMEGUI_H
#define VOLUMEGUI_H

#include <gtkmm.h>
#include <queue>

#include "VolumeConfig.h"
#include "VolumeDialog.h"
#include "VolumeGuiState.h"

#include "Pline/Pline.h"

class DispatchData;

class VolumeGui
{
public:
	VolumeGui(VolumeConfig *poVolumeConfig);
	virtual ~VolumeGui();

	VolumeGuiState* getVolumeGuiState() {return gpoVolumeGuiState;}
	VolumeConfig& getVolumeConfig() {return goVolumeConfig;}
	
	// thread won't be executed if bLock is true
	// ex: call before deleting, return false if thread is running
	bool lockThread(const bool bLock = true); 
	
protected:	
	// VolumeConfig and gui
	VolumeConfig&		goVolumeConfig; // Initial volume config
	VolumeGuiState*		gpoVolumeGuiState;
	
	Glib::Dispatcher	goDispatcher;
	
	// Thread
	bool canStartPline();
	bool lockStartPline();
	
	bool isRunning();
	void launchThread();
	virtual void threadFun() = 0;
	
	// Dispatch
	DispatchData *createDispatchData();
	
	virtual void onDispatcher();
	
	virtual void onJoin();
	virtual void onPlineAskData(Pline::Pline* poPline);
	virtual void onShowDialog(VolumeDialog::tConfig tConfigDialogItem);
	virtual void onUpdateVolume();
	
private:	
	// Thread
	Glib::Thread* gpoThread;
	
	Glib::Mutex		gmtxThread;
	Glib::Mutex		gmtxLockThread;
	bool			gbLockThread;	
	Glib::Mutex		gmtxLockStartPline;
	bool 			gbStartPlineLocked;
	
	void launchThreadIdle();
	void join();
};
	
#endif /* VOLUMEGUI_H */
