/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VOLUMEMOUNTGUI_H
#define VOLUMEMOUNTGUI_H

#include <gtkmm.h>
#include <queue>
#include <vector>

#include "VolumeGui.h"
#include "Pline/ClonePline.h"
#include "Pline/EnlargeVolumePline.h"
#include "Pline/InitVolumePline.h"
#include "Pline/MkDiskImagePline.h"
#include "Pline/MkVolumePline.h"
#include "Pline/MountPline.h"
#include "Pline/ProcessBasePline.h"

class VolumeMountGui : public VolumeGui
{
public:
	VolumeMountGui(VolumeConfig *poVolumeConfig);
	virtual ~VolumeMountGui();
	
	// disk usage
	void calcDiskUsage();
	
	// data in use
	inline std::vector<Pline::MountPline::tDumpStatus>& getPlineStatus() {return gvecPlineStatus;}
	inline const Glib::ustring& getFSys() {return gstFSys;}
	inline const Glib::ustring& getCipher() {return gstCipher;}
	inline const Glib::ustring& getDiskSize() {return gstDiskSize;}
	inline const Glib::ustring& getDiskAvailableSpace() {return gstDiskUsedSpace;}
	inline const Glib::ustring& getDiskUsedSpace() {return gstDiskAvailableSpace;}
	
	bool isInUse(bool *pbMounted = 0); // volume in use
	
	//
	// call only from main thread
	//
	
	// cancel operation
	bool cancel();
	
	// create volume
	bool createDiskImage(	const unsigned long long ullVolumeSize,
							const bool bUseFullAllocation,
							const Glib::ustring& stCipher = "",
							const Glib::ustring& stCBC = "",
							const Glib::ustring& stKeySize = "");
	
	bool createVolume(	const Glib::ustring& stCipher = "",
						const Glib::ustring& stCBC = "",
						const Glib::ustring& stKeySize = "");
						
	// clone
	bool clone();
	
	// enlarge
	bool enlargeVolume(const unsigned long long ullEnlargeSize);
	
	// initialize
	bool initVolume(VolumeConfig* poNewVolumeConfig,
					const Glib::ustring& stCipher = "",
					const Glib::ustring& stCBC = "",
					const Glib::ustring& stKeySize = "");
	
	// mount
	bool eject();
	bool mount();
	
	void updateVolumeStatus();
	void updatePline();
	
	inline sigc::signal<void, bool>& signal_mounted() {return m_signal_mounted;}
	
protected:
	virtual void onJoin();
	virtual void onUpdateVolume();
	
	virtual void threadFun();
	
private:
	std::vector<Pline::MountPline::tDumpStatus> gvecPlineStatus;
	Glib::ustring gstFSys;
	Glib::ustring gstCipher;
	
	// Disk usage
	Glib::ustring gstDiskSize;
	Glib::ustring gstDiskUsedSpace;
	Glib::ustring gstDiskAvailableSpace;
	
	// Mount/Eject
	bool mountEject(const bool bReqMount);

	// Used by initvolume processpline
	VolumeConfig *gpoNewPlineVolumeConfig;
	
	// Signals
	bool gbSigResultCode;
	bool gbSigMount;
	bool gbSigEject;
	
	sigc::signal<void, bool>  m_signal_mounted;
	
	// Pline
	Pline::ProcessBasePline*  	gpoProcessPline;
	Glib::Mutex					gmtxPline;
	
	Pline::ClonePline*  			gpoClonePline;
	Pline::EnlargeVolumePline* 		gpoEnlargeVolumePline;
	Pline::InitVolumePline* 		gpoInitVolumePline;
	Pline::MkDiskImagePline* 		gpoMkDiskImagePline;
	Pline::MkVolumePline* 			gpoMkVolumePline;
	
	Pline::MountPline*  gpoMountPline;
	Glib::Mutex			gmtxMountPline;
	bool 				gbMountPlineBusy;
	
	void connectPlineSignals(Pline::Pline *poPline);
	
	void onPlineReqData();
	void onPlineRunCompleted(const bool bSuccess);
	void onPlineUpdateReq();
	
	void updateMountPlineStatus(); // calc disk usage and dump status
	void updateVolumeGuiState(const bool bPlineMounted);
};
	
#endif /* VOLUMEMOUNTGUI_H */
